/*
                      Blommers IT - Webserver Library
    Copyright (C) 2002-2007 Blommers IT, Rick Blommers. All Rights Reserved.
    -----------------------------------------------------------------------

    This program is free software; you can redistribute it and/or modify it
    under the terms of the  Blommers IT - Free Software License Version 1.0
    as published by Blommers IT;  either version 1 of the License,   or (at
    your option) any later version.
    This program is  distributed in the  hope that  it will be useful,  but
    WITHOUT  ANY  WARRANTY;    without  even  the  implied  warranty     of
    MERCHANTABILITY or  FITNESS FOR A PARTICULAR PURPOSE.  See  the license
    for more details.

    You should have  received  a copy of the   Blommers IT -  Free Software
    License along with this program; if not, write to
            Blommers IT, de Amert 124,
            5462 GH, Veghel, The Netherlands
   or request for the license by sending an e-mail to info@blommers-it.nl
   or visit the website: http://www.blommers-it.nl/
*/

package com.blommersit.httpd;

import java.util.*;

/**
 * This class is used for managing aliases. An alias is an automatic conversion
 * from a GET request to another name. It renames the GET to the given
 * location just before trying to find de HttpHandlers, Resources or Files .
 * <BR/>
 * NOTE: this method sends a 301: Moved permanently to the client if an alias matches
 * <p>
 * <b>
 * NOTE2: (since V1.03)
 * </b>
 * You should use Routes for making 'aliases' for HttpHandlers!
 * </p>
 *
 * @todo In the future this method should true replace the get prefix and not do a 301
 * @author <a href='mailto:rick@blommers-it.nl'>Rick Blommers</a>
 */

public class AliasManager
{
////////////////////////////////////////////////////////////////////////////////
// data members
////////////////////////////////////////////////////////////////////////////////

    /** this hashmap contains all aliasses */
    HashMap aliasmap = new HashMap();

////////////////////////////////////////////////////////////////////////////////
// construction / initialisation
////////////////////////////////////////////////////////////////////////////////

    /**
     * default constructor for the alias manager
     */
    public AliasManager()
    {
    }

////////////////////////////////////////////////////////////////////////////////
// data management
////////////////////////////////////////////////////////////////////////////////

    /**
     * add an alias
     * @param get the prefix of the get method that is an alias
     * @param result the true location of the alias
     */
    public void addAlias( String get, String result )
    {
        aliasmap.put(get, result);
    }

    /**
     * this method retrieves an alias
     * @param get the get value
     * @param def the location to return if the alias isn't found
     * @return the aliased location or def if not found
     */
    public String getAlias( String get, String def )
    {
        if( !aliasmap.containsKey(get) ) return def;
        return (String)aliasmap.get(get);
    }

////////////////////////////////////////////////////////////////////////////////
// this method checks if we should send a 'moved' permanently message
////////////////////////////////////////////////////////////////////////////////

	/**
	 * check the allias. If found send a 301, moved permanently
	 * @param request the request object
	 * @param response the response object
	 * @return true if redirected to another location
	 */
	public boolean redirectLocationIfRequired( HttpRequest request, HttpResponse response)
	{
		String alias = getAlias( request.getLocation(), null );
		if( alias == null ) return false;
		response.setHeaderBegin( "HTTP/1.1 301 Moved Permanently", 301 );
		response.setHeader("location:",alias);
		response.println("<html><body>Moved permanently to <a href='"+alias+ "'>"+alias+"</a></body></html>");
		try
		{
			response.flush();
			return true;
		}
		catch( Exception e  )
		{
			// Since V1.03
			// if the flush throws an exception, this usually means the other
			// side has disconnected the connection, which isn't a real problem!
			return true;
		}
	}

}
