/*
                      Blommers IT - Webserver Library
    Copyright (C) 2002-2007 Blommers IT, Rick Blommers. All Rights Reserved.
    -----------------------------------------------------------------------

    This program is free software; you can redistribute it and/or modify it
    under the terms of the  Blommers IT - Free Software License Version 1.0
    as published by Blommers IT;  either version 1 of the License,   or (at
    your option) any later version.
    This program is  distributed in the  hope that  it will be useful,  but
    WITHOUT  ANY  WARRANTY;    without  even  the  implied  warranty     of
    MERCHANTABILITY or  FITNESS FOR A PARTICULAR PURPOSE.  See  the license
    for more details.

    You should have  received  a copy of the   Blommers IT -  Free Software
    License along with this program; if not, write to
            Blommers IT, de Amert 124,
            5462 GH, Veghel, The Netherlands
   or request for the license by sending an e-mail to info@blommers-it.nl
   or visit the website: http://www.blommers-it.nl/
*/

package com.blommersit.httpd;

import java.io.*;
import java.util.*;

/**
 * Deze klasse bevat een aantal HTTP Utils, waarmee heel eenvoudig
 * HTTP Server responses gegenereerd kunnen worden
 * @author <a href='mailto:rick@blommers-it.nl'>Rick Blommers</a>
 */
public class HttpUtils
{
////////////////////////////////////////////////////////////////////////////////
// constants
////////////////////////////////////////////////////////////////////////////////

    /** Status-Code 200: OK. */
    public final static int HTTP_OK = 200;
    /** Status-Code 301: Moved Permanently*/
    public final static int HTTP_MOVED_PERMANENTLY = 301;
    /** Status-Code 400: Bad Request. */
    public final static int HTTP_BAD_REQUEST = 400;
    /** Status-Code 404: Not Found. */
    public final static int HTTP_NOT_FOUND = 404;
    /** Status-Code 405: Method Not Allowed. */
    public final static int HTTP_BAD_METHOD = 405;
    /** Status-Code 411: Length Required. */
    public final static int HTTP_LENGTH_REQUIRED = 411;
    /** Status-Code 500: Internal Server Error. */
    public final static int HTTP_INTERNAL_ERROR = 500;

////////////////////////////////////////////////////////////////////////////////
// construction / initialisation
////////////////////////////////////////////////////////////////////////////////

    /** default constructor */
    public HttpUtils()
    {
    }

////////////////////////////////////////////////////////////////////////////////
// error resposne
////////////////////////////////////////////////////////////////////////////////

	/**
	 * This method converts an error number to a name.
	 * When an invalid number is assigned this method always returns
	 * "Internal Server Errror"!
	 * @param writer DataOutputStream
	 * @param value int
	 * @return int
	 * @since V1.03
	 */
	public static String getHttpResultText( int code )
	{
		switch( code )
		{
			case HTTP_OK:				return "OK";
			case HTTP_BAD_REQUEST:		return "Bad Request";
			case HTTP_NOT_FOUND:		return "Not Found";
			case HTTP_BAD_METHOD:		return "Method Not Allowed";
			case HTTP_LENGTH_REQUIRED:	return "Length Required";
			default:
			case HTTP_INTERNAL_ERROR:	return "Internal Server Error";
		}
	}




	/**
	 * this method sends an error message to client and returns the error code
	 * This need to be done not this way
	 * @todo this method need to dissapear an be replaced by 'custom' error
	 * handlers, sepcified in the HttpServerSettings
	 * @param writer the writer
	 * @param value the http result.
	 * @throws IOException yep it can happen
	 * @return the HTTP_ result code
	 * @deprecated, please use the HttpResponse setHttpResult version! or the
	 * 			sendErrorMessage with first parameter Respones!
	 */
	public static int sendErrorMessage( DataOutputStream writer , int value ) throws IOException
	{
		String message = "HTTP/1.1 " + value + " ";
		message += getHttpResultText( value );
		String errorMessage = message;
		message += "\r\n";
		message += "Date: " + (new Date()).toGMTString() + "\r\n";
		message += "Server: Blommers IT Webserver Library\r\n";
		message += "Connection: close\r\n";
		message += "Content-Type: text/html; charset=iso-8859-1\r\n";
		message += "\r\n";
		writer.writeBytes(message);
		writer.writeBytes("<html><head><title>"+errorMessage+"</title></head><body><font color='red'>"+errorMessage+"</font></body></html>\r\n\r\n");
		writer.flush();
		return value;
	}

	/**
	 * this method sends an error message to client and returns the error code
	 * This need to be done not this way
	 * @todo this method need to dissapear an be replaced by 'custom' error
	 * handlers, specified in the HttpServerSettings
	 *
	 * This method is also temporary, we need to build in Custom error docs
	 * in the near future
	 *
	 * @param response the http response
	 * @param httpResult the resultcode
	 * @throws IOException yep it can happen
	 * @return the HTTP_ result code
	 */
	public static int sendErrorMessage( HttpResponse response, int httpResult  ) throws IOException
	{
		response.setHttpResult( httpResult );
		String msg = getHttpResultText( httpResult );
		response.print( "<html><head><title>"+msg+"</title></head><body><font color='red'>"+msg+"</font></body></html>\r\n\r\n" );
		response.flush();
		return httpResult;
	}


////////////////////////////////////////////////////////////////////////////////
// parsing help
////////////////////////////////////////////////////////////////////////////////

	/**
	 * this method adds a variable to a HashMap. If the key already exists
	 * the current value is converted to an arraylist.
	 * @param id the variable name
	 * @param value the value
	 * @since V1.03
	 */
	public static void addToHashMap( HashMap map, String id, String value )
	{
		if( map.containsKey( id ) )
		{
			Object o = map.get( id );
			if( o instanceof ArrayList )
			{
				((ArrayList)o).add( value );
			}
			else
			{
				ArrayList arr = new ArrayList();
				arr.add( o );
				arr.add( value );
				map.put( id, arr );
			}
		}
		else
		{
			map.put( id, value );
		}
	}



	/**
	 * this method parses a raw put or get string an puts it in a hashmap
	 * @param options the put/get string. example: variable=x&variable=y
	 * @param output the Hashmap to fill with the given variables. Every
	 *              value of the options are being URLDecoded.
	 */
	public static void parse_str( String options, HashMap output )
	{
		parse_str( options, output, false );
	}

	/**
	 * this method parses a raw put or get string an puts it in a hashmap
	 * @param options the put/get string. example: variable=x&variable=y
	 * @param output the Hashmap to fill with the given variables. Every
	 *              value of the options are being URLDecoded.
	 * @param makeArrayList set this method to true, to make an arraylist if
	 * 			a method is used multiple times
	 * @since V1.03
	 */
	public static void parse_str( String options, HashMap output, boolean makeArrayList )
	{
		parse_str( options, output, makeArrayList, "&" );
	}

	/**
	 * this method parses a raw put or get string an puts it in a hashmap
	 * @param options the put/get string. example: variable=x&variable=y
	 * @param output the Hashmap to fill with the given variables. Every
	 *              value of the options are being URLDecoded.
	 * @param makeArrayList set this method to true, to make an arraylist if
	 * 			a method is used multiple times
	 * @param optionSeperator added an option seperator so this method can also
	 * 			be used to parse the cookies
	 * @since V1.03
	 */
	public static void parse_str( String options, HashMap output, boolean makeArrayList, String optionSeperator )
	{
		String items[] = Utils.explode(optionSeperator,options);
		if( items != null && items.length  > 0 )
		{
			for( int i=0; i < items.length; i++ )
			{
				int equalPos = items[i].indexOf('=');
				if (equalPos != 0)
				{
					// parse the item in a key and a value
					String key = "", value = "";
					if ( equalPos > 0 )
					{
						key = items[i].substring(0,equalPos).trim();
						try
						{
							value = java.net.URLDecoder.decode( items[i].substring(equalPos+1), "UTF-8" );
						}
						catch(UnsupportedEncodingException e)
						{
							e.printStackTrace();
						}

					}
					else key = items[i];

					// add te item in an arraylist
					if( makeArrayList )
					{
						addToHashMap( output, key, value );
					}
					// add the value
					else
					{
						output.put(key, value);
					}
				}
			}
		}
	}


}
