/*
                      Blommers IT - Webserver Library
    Copyright (C) 2002-2007 Blommers IT, Rick Blommers. All Rights Reserved.
    -----------------------------------------------------------------------

    This program is free software; you can redistribute it and/or modify it
    under the terms of the  Blommers IT - Free Software License Version 1.0
    as published by Blommers IT;  either version 1 of the License,   or (at
    your option) any later version.
    This program is  distributed in the  hope that  it will be useful,  but
    WITHOUT  ANY  WARRANTY;    without  even  the  implied  warranty     of
    MERCHANTABILITY or  FITNESS FOR A PARTICULAR PURPOSE.  See  the license
    for more details.

    You should have  received  a copy of the   Blommers IT -  Free Software
    License along with this program; if not, write to
            Blommers IT, de Amert 124,
            5462 GH, Veghel, The Netherlands
   or request for the license by sending an e-mail to info@blommers-it.nl
   or visit the website: http://www.blommers-it.nl/
*/

package com.blommersit.httpd;

import java.io.*;

import java.util.*;
import java.text.*;

/**
 * General Utils
 * @author <a href='mailto:rick@blommers-it.nl'>Rick Blommers</a>
 */
public final class Utils
{

    /**
     * this method safely parses a long
     * @param value the value to parse
     * @param radix the radix  (use 10 for decimal, 16 for hexadecimal)
     * @param def the default value to return on a parse exception
     * @return the parsed string ( or def if a parse exception occured)
     */
    public static long parseLong( String value, int radix, long def )
    {
        try
        {
            return Long.parseLong(value, radix);
        }
        catch(Exception e )
        {
        }
        return def;
    }

    /**
     * this method safely parses an integer
     * @param value the string value to parse
     * @param radix the radix  (use 10 for decimal, 16 for hexadecimal)
     * @param def the default value to return on a parse exception
     * @return the parsed string ( or def if a parse exception occured)
     */
    public static int parseInt( String value, int radix, int def )
    {
        try
        {
            return Integer.parseInt(value.trim(),radix);
        }
        catch(Exception e)
        {
        }
        return def;
    }

	/**
	 * this method splits a string into pieces. and returns the result in
	 * a string array. For example:<br/>
	 * String[] result = explode("+","value1+value2+value3");<br/>
	 * returns:<br/>
	 * result[0] = "value1"; result[1] = "value2"; result[2]="value3";
	 * </pre>
	 * @param items the string with items.
	 * @return a string array with all strings
	 */
	public static String[] explode( String delim, String items )
	{
/** @todo optimize this method */
		//SPLIT Doesn't return empty substrings :(
		//* @param delim the seperator. WARNING Since V1.03 this is a regular expressions!!
		// return items.split( delim );
		//StringTokenizer st = new StringTokenizer( items, delim );
		// the vector that's going to recieve all elements
		ArrayList v = new ArrayList();

		// the current token and the 'delim' reader
		StringBuffer curToken = new StringBuffer();
		StringBuffer delimReader = new StringBuffer();

		// read all characters
		int lengte = items.length();
		int delimLengte = delim.length();
		for( int i=0; i < lengte; i++ )
		{
			// get the character
			char c = items.charAt(i);

			// check if it could be the delimiter
			delimReader.setLength(0);
			if( c == delim.charAt(0) )
			{
				// check the whole delimiter
				boolean equals=true;
				int j=1;
				for( ; (j < delimLengte) && (( j+i)< lengte ) ; j++ )
				{
					if ( items.charAt(i+j) != delim.charAt(j ) )
					{
						equals=false;
						break;
					}
				}

				// delimiter found !!
				if ( j==delimLengte && equals )
				{
					i = i + delimLengte-1;
					v.add( curToken.toString() );
					curToken.setLength(0);
				}
				else curToken.append(c);
			}
			else curToken.append(c);
		}
		v.add( curToken.toString() );
		curToken.setLength(0);

		String[] sa = new String[ v.size() ];
		for( int i=0; i < v.size(); i++ ) sa[ i ] = v.get(i).toString();
		return sa;
	}


    /**
     * This method concats two optionlists and returns them in a string
     * @param optionList1 the first optionlist
     * @param optionList2 the second optionlist
     * @return the concatted optionlist
     */
    public static String concatOptionList( String optionList1, String optionList2 )
    {
        String result = "";
        if( optionList1 != null ) result = optionList1.trim();
        if( optionList2 != null )
        {
            optionList2 = optionList2.trim();
            if( optionList2.length() > 0 )
            {
                if( result.length() > 0 ) result += ",";
                result += optionList2;
            }
        }
        return result;
    }

    /**
     * This method parses an option list. An optionlist is a list of options placed
     * in a single string:
     * "optionname=value,optionname2="value2"
     * @param map the map that needs to recieve the options. NOTE: if the option
     *              already exists it will be overwriten
     * @param optionList The optionList string
     */
    public static void parseOptionList( Map map, String optionList ) throws ParseException
    {
        // parse the complete string
        int length = optionList.length();
        StringBuffer key=new StringBuffer(50);
        StringBuffer value=new StringBuffer(50);
        boolean valueMode=false;
        for( int i=0; i < length; i++ )
        {
            // decide what to do
            char c = optionList.charAt(i);

            // ignore spaces
            if( Character.isWhitespace(c) ) continue;

            // is it a 'quote'
            if( c == '\"' || c=='\'' )
            {
                char lastQuoteChar=c;

                if( !valueMode ) throw new ParseException("A 'quote' cannot be used in the key!",i);
                if( value.length() > 0 ) throw new ParseException("Only at the start of a value, it's allowed to start a string!",i);

                // read until the end of the string
                i++;
                while( i < length )
                {
                    c = optionList.charAt(i);
                    if( c ==lastQuoteChar ) break;
                    // escape characters
                    if( c =='\\' && (( i+1)<length ) )
                    {
                        i++;
                        c = optionList.charAt(i);
                        if( c=='\"' ) value.append('\"');
                        else if( c=='\'' ) value.append('\'');
                        else if( c=='\\' ) value.append('\\');
                        else if( c=='b' ) value.append('\b');
                        else if( c=='t' ) value.append('\t');
                        else if( c=='r' ) value.append('\r');
                        else if( c=='n' ) value.append('\n');
                        else if( c=='f' ) value.append('\f');
                        else throw new ParseException("Invalid escape-sequence in value string",i );
                    }
                    else value.append(c);
                    i++;
                }
            }
            // equals symbol
            else if ( c=='=' )
            {
                if( valueMode ) throw new ParseException("Expected a value but found a '=' at "+key, i);
                valueMode = true;
            }
            // next option ??
            else if ( c == ',' )
            {
                map.put(key.toString(),value.toString());
                key.setLength(0);
                value.setLength(0);
                valueMode = false;
            }
            // append the character
            else
            {
                if( valueMode ) value.append(c);
                else key.append(c);
            }
        }

        // still a key available ?
        if( key.length() > 0 )
        {
            map.put(key.toString(),value.toString());
        }
    }


    /**
     * This method returns a string with options. This string is parsable with
     * parse optionlist
     * @param options the map with options. NOTE de keys of this map
     *              need to be string. The keynames may NOT contain spaces, returns, equal-sign or comma's
     *              The value also needs to be a string or a Number
     *              Special characters are automaticly being escaped.
     * @return a single string of the form  key="string value\nNext line",key1="value";
     */
    public static String generateOptionsList( Map options )
    {
        StringBuffer buf = new StringBuffer(200);
        Iterator it = options.keySet().iterator();
        while( it.hasNext() )
        {
            String key = (String)it.next();
            Object oval = options.get(key);
            if( buf.length() > 0 ) buf.append(',');
            buf.append(key);
            buf.append('=');

            String s = oval.toString();
            buf.append('\"');
            for( int i=0; i < s.length(); i++ )
            {
                char c = s.charAt(i);
                if(  c=='\"' || c=='\'' || c=='\\')
                {
                    buf.append('\\');
                    buf.append(c);
                }
                else if( c == '\r' ) buf.append("\\\r");
                else if( c == '\n' ) buf.append("\\\n");
                else if( c == '\t' ) buf.append("\\\t");
                else if( c == '\b' ) buf.append("\\\b");
                else if( c == '\f' ) buf.append("\\\f");
                else buf.append(c);
            }
            buf.append('\"');
        }
        return buf.toString();
    }

    /**
     * makes a string by appending the char to the end or to the beginning of the string
     * @param in the input string
     * @param length the length this string needs to be
     * @param append the character that should be appended
     * @param trailing should we put characters to the beginning (false) or to the end (true)
     */
    public static String makeString( String in, int length, char append, boolean trailing )
    {
        if ( !trailing )
        {
            if ( in == null ) return null;
            int sl = in.length();
            if ( sl >= length ) return in;

            StringBuffer suit = new StringBuffer(length);
            int l = length - sl;
            for ( int i = 0; i < l; i++ ) suit.append( append );
            suit.append( in );
            return suit.toString();
        }

        StringBuffer s = new StringBuffer( in );
        while( s.length() < length )
        {
            s.append(append);
        }
        return s.toString();
    }


	/**
	 * timer stack
	 * @since V1.03
	 */
	static Stack timeStack = new Stack();

	/**
	 * For performing timing operations
	 * @since V1.03
	 */
	public static void startTimer()
	{
		timeStack.add( new Long( System.currentTimeMillis() ) );
	}

	/**
	 * For performing timing operations
	 * @since V1.03
	 */
	public static String stopTimer( )
	{
		if( timeStack.size() > 0 )
		{
			long stopValue = System.currentTimeMillis();
			long startValue = ((Long)timeStack.pop()).longValue();
			long diff = stopValue - startValue;
			String value = "";
			value += ( diff / 1000 / 60 ) % 60 + "m ";
			value += ( diff / 1000 ) % 60 + "s ";
			value += ( diff % 1000 ) + "ms";
			return value;
		}
		return "** missing starttimer **";
	}


	/**
	 * Returns an inputstream as an array as bytes.
	 * @param stream InputStream the inputstream to read
	 * @return byte[] the array of bytes
	 * @since V1.03
	 */
	public static byte[] getStreamAsBytes( InputStream stream ) throws IOException
	{
		byte[] result = new byte[0];
		byte[] buffer = new byte[4096];	// 4k per read
		int readSize = 0;
		while( ( readSize = stream.read( buffer ) ) > 0 )
		{
			int totalSize = result.length + readSize;
			byte[] tmpResult = result;
			result = new byte[ totalSize ];
			System.arraycopy( tmpResult, 0, result, 0, tmpResult.length );
			System.arraycopy( buffer, 0, result, tmpResult.length, readSize );
		}
		return result;
	}


}
