/*
					  Blommers IT - Webserver Library
	Copyright (C) 2002-2007 Blommers IT, Rick Blommers. All Rights Reserved.
	-----------------------------------------------------------------------

	This program is free software; you can redistribute it and/or modify it
	under the terms of the  Blommers IT - Free Software License Version 1.0
	as published by Blommers IT;  either version 1 of the License,   or (at
	your option) any later version.
	This program is  distributed in the  hope that  it will be useful,  but
	WITHOUT  ANY  WARRANTY;    without  even  the  implied  warranty     of
	MERCHANTABILITY or  FITNESS FOR A PARTICULAR PURPOSE.  See  the license
	for more details.

	You should have  received  a copy of the   Blommers IT -  Free Software
	License along with this program; if not, write to
			Blommers IT, de Amert 124,
			5462 GH, Veghel, The Netherlands
   or request for the license by sending an e-mail to info@blommers-it.nl
   or visit the website: http://www.blommers-it.nl/
*/
package com.blommersit.httpd.interceptor;

import java.util.*;
import com.blommersit.httpd.*;

/**
 * This class is used to manage all response interceptors.
 * <p>
 * An interceptor is used to intercept all requests to the webserver. The
 * response interceptors is called just before the headers are sent. This way
 * it's possible to adjust headers for a certain request.
 * </p>
 *
 * @author <a href='mailto:rick@blommers-it.nl'>Rick Blommers</a>
 * @since V1.03
 */
public class HttpInterceptorManager
{
////////////////////////////////////////////////////////////////////////////////
// Data Members
////////////////////////////////////////////////////////////////////////////////

	/** the current response interceptors */
	ArrayList httpResponseInterceptors = new ArrayList();

////////////////////////////////////////////////////////////////////////////////
// Listener installing interface
////////////////////////////////////////////////////////////////////////////////

	/** adds an interceptor */
	public void addInterceptor( HttpResponseInterceptor interceptor )
	{
		this.httpResponseInterceptors.add( interceptor );
	}

	/** removes an interceptor */
	public void removeInterceptor( HttpResponseInterceptor interceptor )
	{
		this.httpResponseInterceptors.remove( interceptor );
	}

////////////////////////////////////////////////////////////////////////////////
// Notification
////////////////////////////////////////////////////////////////////////////////

	/**
	 * This method gives all interceptors the possibility to intercept the
	 * respones headers just before the headers are sent!
	 */
	public void interceptWriteHeaders( HttpServerThread thread, HttpRequest request, HttpResponse response )
	{
		for( int i=0; i < httpResponseInterceptors.size(); i++ )
		{
			HttpResponseInterceptor interceptor = (HttpResponseInterceptor)httpResponseInterceptors.get( i );
			interceptor.interceptWriteHeaders( thread, thread.getServer(), request, response );
		}
	}


}

