/*
					  Blommers IT - Webserver Library
	Copyright (C) 2002-2007 Blommers IT, Rick Blommers. All Rights Reserved.
	-----------------------------------------------------------------------

	This program is free software; you can redistribute it and/or modify it
	under the terms of the  Blommers IT - Free Software License Version 1.0
	as published by Blommers IT;  either version 1 of the License,   or (at
	your option) any later version.
	This program is  distributed in the  hope that  it will be useful,  but
	WITHOUT  ANY  WARRANTY;    without  even  the  implied  warranty     of
	MERCHANTABILITY or  FITNESS FOR A PARTICULAR PURPOSE.  See  the license
	for more details.

	You should have  received  a copy of the   Blommers IT -  Free Software
	License along with this program; if not, write to
			Blommers IT, de Amert 124,
			5462 GH, Veghel, The Netherlands
	or request for the license by sending an e-mail to info@blommers-it.nl
	or visit the website: http://www.blommers-it.nl/
*/

package com.blommersit.httpd.log;

import com.blommersit.httpd.*;

/**
 * You can implement this listener to to add a server log listener.
 * The server will notify the listeners with events that take place while
 * handling requests etc.
 * <p>
 * The listener can do with the messages wathever they want, like writing
 * it to the console or logging it in a log file or perhaps writing it in
 * a database. For the developers who like to recieve spam I would suggest a
 * Mail log handler...
 * </p>
 *
 * @author <a href='mailto:rick@blommers-it.nl'>Rick Blommers</a>
 * @since V1.03
 */
public interface HttpServerLogListener
{
	/**
	 * the different debug levels
	 * DEBUG_LEVEL_NONE = No debug messages
	 * DEBUG_LEVEL_MIN = minimal amount of debug messages
	 * DEBUG_LEVEL_MEDIUM = medium amount of debug messages
	 * DEBUG_LEVEL_MAX = medium amount of debug messages
	 */
	public final static int DEBUB_LEVEL_NONE = 0,
				DEBUG_LEVEL_MIN = 1,
				DEBUG_LEVEL_MEDIUM = 2,
				DEBUG_LEVEL_MAX = 3
				;

	/**
	 * this log method is called when the server wants to log the request mapping
	 * @param thread the server thread currently handling the request
	 * @param request the http request
	 * @param response the http response
	 * @param outType the 'matches' response output type. This is:<ul>
	 * 			<li>"java", for java class file </li>
	 * 			<li>"jar", for a jar resource</li>
	 * 			<li>"file", for the filesystem document root</li>
	 * 			</ul>
	 * @param pathOut the output path, depending on the matched output type
	 * 			will be a full classname or pathname.
	 */
	public void logRequest( HttpServerThread thread, HttpRequest request, HttpResponse response, String outputType, String pathOut );

	/**
	 * logs an error message
	 * @param thread the server thread currently handling the request
	 * @param request the http request
	 * @param response the http response
	 * @param errorMessage the error message
	 */
	public void logError( HttpServerThread thread, HttpRequest request, HttpResponse response, String errorMessage );

	/**
	 * logs a redirect
	 * @param thread the server thread currently handling the request
	 * @param request the http request
	 * @param response the http response
	 * @param redirectTo the redirected to path
	 */
	public void logRedirect( HttpServerThread thread, HttpRequest request, HttpResponse response, String redirectTo );


	/**
	 * Logs a debug message.
	 * @param thread the server thread currently handling the request
	 * @param request the http request. WARNING: This member can be NULL !!
	 * @param response the http response. WARNING: This member can be NULL !!
	 * @param level the debug level
	 * @param message the debug message
	 */
	public void logDebugMessage( HttpServerThread thread, HttpRequest request, HttpResponse response, int level, String message );

}
