/*
					  Blommers IT - Webserver Library
	Copyright (C) 2002-2007 Blommers IT, Rick Blommers. All Rights Reserved.
	-----------------------------------------------------------------------

	This program is free software; you can redistribute it and/or modify it
	under the terms of the  Blommers IT - Free Software License Version 1.0
	as published by Blommers IT;  either version 1 of the License,   or (at
	your option) any later version.
	This program is  distributed in the  hope that  it will be useful,  but
	WITHOUT  ANY  WARRANTY;    without  even  the  implied  warranty     of
	MERCHANTABILITY or  FITNESS FOR A PARTICULAR PURPOSE.  See  the license
	for more details.

	You should have  received  a copy of the   Blommers IT -  Free Software
	License along with this program; if not, write to
			Blommers IT, de Amert 124,
			5462 GH, Veghel, The Netherlands
	or request for the license by sending an e-mail to info@blommers-it.nl
	or visit the website: http://www.blommers-it.nl/
*/

package com.blommersit.httpd.log;

import java.util.*;

import com.blommersit.httpd.*;

/**
 * The log listener manager manages al log listeners. You can add log managers
 * to the logListenerManager that will recieve events from the webserver.
 * This way it's possible to recieve request information from the webserver
 *
 * <p>
 * NOTE: You must increase the debug level to recieve debug messages!
 * </p>
 *
 * @author <a href='mailto:rick@blommers-it.nl'>Rick Blommers</a>
 * @since V1.03
 */
public class LogListenerManager
{
////////////////////////////////////////////////////////////////////////////////
// Data members
////////////////////////////////////////////////////////////////////////////////

	/** the array with log listeners */
	ArrayList listeners = new ArrayList();

	/** the current debug level */
	int currentDebugLevel = HttpServerLogListener.DEBUB_LEVEL_NONE;

////////////////////////////////////////////////////////////////////////////////
// Listener installing interface
////////////////////////////////////////////////////////////////////////////////

	/** adds a listeners */
	public void addListener( HttpServerLogListener listener )
	{
		this.listeners.add( listener );
	}

	/** removes a listener */
	public void removeListener( HttpServerLogListener listener )
	{
		this.listeners.remove( listener );
	}

////////////////////////////////////////////////////////////////////////////////
// Debugging options
////////////////////////////////////////////////////////////////////////////////

	/**
	 * this method sets the debug level
	 * return value is one of the HttpServerLogListener.DEBUG_LEVEL_*
	 * constants!
	 */
	public void setDebugLevel( int level )
	{
		this.currentDebugLevel = level;
	}

	/**
	 * returns the current debug level.
	 * return value is one of the HttpServerLogListener.DEBUG_LEVEL_*
	 * constants!
	 */
	public int getDebugLevel()
	{
		return currentDebugLevel;
	}

////////////////////////////////////////////////////////////////////////////////
// Internal Methods
////////////////////////////////////////////////////////////////////////////////

	/**
	 * this method notifies all listeners a page has been requested
	 */
	public synchronized void notify_logRequest( HttpServerThread thread, HttpRequest request, HttpResponse response, String outputType, String pathOut )
	{
		for( int i = 0; i < this.listeners.size(); i++ )
		{
			HttpServerLogListener listener = (HttpServerLogListener)listeners.get( i );
			listener.logRequest( thread, request, response, outputType, pathOut );
		}
	}

	/**
	 * notifies all listeners an error occured
	 */
	public synchronized void notify_logError( HttpServerThread thread, HttpRequest request, HttpResponse response, String errorMessage )
	{
		for( int i = 0; i < this.listeners.size(); i++ )
		{
			HttpServerLogListener listener = (HttpServerLogListener)listeners.get( i );
			listener.logError( thread, request, response, errorMessage );
		}
	}

	/**
	 * notifies all listeners a redirect happend !
	 */
	public synchronized void notify_logRedirect( HttpServerThread thread, HttpRequest request, HttpResponse response, String redirectTo )
	{
		for( int i = 0; i < this.listeners.size(); i++ )
		{
			HttpServerLogListener listener = (HttpServerLogListener)listeners.get( i );
			listener.logRedirect( thread, request, response, redirectTo );
		}
	}

	/**
	 * notifies all listeners the given debug message
	 */
	public void notify_logDebugMessage( HttpServerThread thread, HttpRequest request, HttpResponse response, int level, String message )
	{
		if( level <= currentDebugLevel )
		{
			for (int i = 0; i < this.listeners.size(); i++) {
				HttpServerLogListener listener = (HttpServerLogListener)
					listeners.get(i);
				listener.logDebugMessage(thread, request, response, level, message);
			}
		}
	}
}
