/*
					  Blommers IT - Webserver Library
	Copyright (C) 2002-2007 Blommers IT, Rick Blommers. All Rights Reserved.
	-----------------------------------------------------------------------

	This program is free software; you can redistribute it and/or modify it
	under the terms of the  Blommers IT - Free Software License Version 1.0
	as published by Blommers IT;  either version 1 of the License,   or (at
	your option) any later version.
	This program is  distributed in the  hope that  it will be useful,  but
	WITHOUT  ANY  WARRANTY;    without  even  the  implied  warranty     of
	MERCHANTABILITY or  FITNESS FOR A PARTICULAR PURPOSE.  See  the license
	for more details.

	You should have  received  a copy of the   Blommers IT -  Free Software
	License along with this program; if not, write to
			Blommers IT, de Amert 124,
			5462 GH, Veghel, The Netherlands
	or request for the license by sending an e-mail to info@blommers-it.nl
	or visit the website: http://www.blommers-it.nl/
*/

package com.blommersit.httpd.log;

import java.text.*;
import java.util.*;
import com.blommersit.httpd.*;

/**
 * A simple systemout log listener, write all log messages to a readable output
 *
 * @author <a href='mailto:rick@blommers-it.nl'>Rick Blommers</a>
 * @since V1.03
 */
public class SystemOutLogListener implements HttpServerLogListener
{

	/** interneral start item */
	private void startLogLine( HttpServerThread thread, HttpRequest request, HttpResponse response, boolean requestInfo )
	{
		SimpleDateFormat formatter = new SimpleDateFormat ("yyyy-MM-dd hh:mm:ss");
		System.out.print( formatter.format( Calendar.getInstance().getTime() ) );
		System.out.print( " " + thread.getName() + " " );
		if( requestInfo )
		{
			System.out.print("[");
			System.out.print(request.getIP());
			System.out.print("]");
			System.out.print(" : " + request.getLocation() + " => ");
			System.out.print("(");
			System.out.print(response.getHeaderResultCode());
			System.out.print(") ");
		}
	}

	/** Simply write the request to the console  */
	public void logRequest( HttpServerThread thread, HttpRequest request, HttpResponse response, String outputType, String pathOut )
	{
		startLogLine( thread, request, response, true );
		System.out.print( outputType );
		System.out.print( " : " );
		System.out.print( "\"" );
		System.out.print( pathOut );
		System.out.println( "\"" );
	}

	/** logs an error message */
	public void logError( HttpServerThread thread, HttpRequest request, HttpResponse response,  String errorMessage )
	{
		startLogLine( thread, request, response, true );
		System.out.print( " | <error> | " );
		System.out.println( errorMessage );
	}

	/**
	 * logs a redirect
	 * @param request  the request object
	 * @param redirectTo the redirected to path
	 * @param result the result
	 */
	public void logRedirect( HttpServerThread thread, HttpRequest request, HttpResponse response, String redirectTo  )
	{
		startLogLine( thread, request, response, true );
		System.out.print( " | <redirect> | " );
		System.out.println( redirectTo );
	}

	/**
	 * Logs a debug message
	 * @param thread the server thread currently handling the request
	 * @param request the http request
	 * @param response the http response
	 * @param message the debug message
	 */
	public void logDebugMessage( HttpServerThread thread, HttpRequest request, HttpResponse response, int level, String message )
	{
		System.out.print( " - " );
		startLogLine( thread, request, response, false );
		for( int i=0; i < level; i++ ) System.out.print( "  " );
		System.out.println(" > " + message);
	}


}
