/*
                      Blommers IT - Webserver Library
    Copyright (C) 2002-2007 Blommers IT, Rick Blommers. All Rights Reserved.
    -----------------------------------------------------------------------

    This program is free software; you can redistribute it and/or modify it
    under the terms of the  Blommers IT - Free Software License Version 1.0
    as published by Blommers IT;  either version 1 of the License,   or (at
    your option) any later version.
    This program is  distributed in the  hope that  it will be useful,  but
    WITHOUT  ANY  WARRANTY;    without  even  the  implied  warranty     of
    MERCHANTABILITY or  FITNESS FOR A PARTICULAR PURPOSE.  See  the license
    for more details.

    You should have  received  a copy of the   Blommers IT -  Free Software
    License along with this program; if not, write to
            Blommers IT, de Amert 124,
            5462 GH, Veghel, The Netherlands
   or request for the license by sending an e-mail to info@blommers-it.nl
   or visit the website: http://www.blommers-it.nl/
*/

package com.blommersit.httpd.security;

import java.util.*;

import com.blommersit.httpd.*;

/**
 * This class represents an access filter. An access filter is an array with
 * rules describing what is allowed and what isn't allowed
 * @author <a href='mailto:rick@blommers-it.nl'>Rick Blommers</a>
 */
public class AccessFilter
{
////////////////////////////////////////////////////////////////////////////////
// data members
////////////////////////////////////////////////////////////////////////////////

    /** this array contains all filter rules */
    ArrayList filterRules = new ArrayList();

    /** the action to take an non-matching rules */
    int defaultAction = AccessFilterRule.ACTION_DENY;


////////////////////////////////////////////////////////////////////////////////
// construction / initialisation
////////////////////////////////////////////////////////////////////////////////

    /**
     * The default constructor for the access filter
     * standard the arraylist contains the following rules:
     * HttpHandler =&gt; Allow All
     * Else =&gt; Deny All
     */
    public AccessFilter()
    {
        addRule(new AccessFilterRule(AccessFilterRule.RESOURCE_TYPE_JAVA_HANDLER,0,0,"",AccessFilterRule.ACTION_ALLOW));
    }


////////////////////////////////////////////////////////////////////////////////
// rule alternation
////////////////////////////////////////////////////////////////////////////////

    /**
     * this method adds a rule at the specified index
     * @param index the index to insert the rule
     * @param rule the rule to add
     */
    public void addRule( int index, AccessFilterRule rule )
    {
        filterRules.add(index,rule);
    }

    /**
     * this method adds a rule to the end of the rules list
     * @param rule the rule to add
     */
    public void addRule( AccessFilterRule rule )
    {
        filterRules.add(rule);
    }

    /**
     * this method removes a rule
     * @param rule the rule to remove
     */
    public void delRule( AccessFilterRule rule )
    {
        filterRules.remove(rule);
    }

    /**
     * this method removes a rule at the given index
     * @param index the rule index to remove
     */
    public void delRule( int index )
    {
        filterRules.remove(index);
    }

    /**
     * this method retrieves a rule at the given index
     * @param index the index of the rule
     * @return the Rule object
     */
    public AccessFilterRule getRule( int index )
    {
        return (AccessFilterRule)filterRules.get(index);
    }

    /**
     * this method returns the number of available rules
     * @return the number of rules in the list
     */
    public int getRuleCount()
    {
        return filterRules.size();
    }

////////////////////////////////////////////////////////////////////////////////
// this method returns the action for the given request
////////////////////////////////////////////////////////////////////////////////

    /**
     * this method returns the action for the given request. The action
     * can be AccessFilterRule.DENY or AccessFilterRule.ALLOW
     * @param type the type of resource
     * @param request the request object
     * @return the AccessFilterRule.DENY or AccessFilterRule.ALLOW constant
     */
    public int getAction( int type, HttpRequest request )
    {
        for( int i=0; i < getRuleCount(); i++ )
        {
            AccessFilterRule rule = getRule(i);
            if( rule.matchesRule(type, request) ) return rule.getAction();
        }
        return defaultAction;
    }

}
